
/******************************** INCLUDES ************************************/
#include <Rolling_Display.h>

/*---------------------------- END OF INCLUDES -------------------------------*/






/***************************** GLOBAL VARIABLES *******************************/


// UART channel selected is UART3
UART_HandleTypeDef huart3;


// To map the status of each LED of the display
uint8_t data_buffer[num_displays][8];
uint8_t display_col_map[num_displays*8];


// Flag variable to track if data is received from Bluetooth
uint8_t uart_rx_flag;


// To store values from Bluetooth data buffer
char bluetooth_data[buffer_size];	// Stores the Bluetooth data buffer
char brightness_val;				// Stores brightness value
char speed_val;						// Stores speed value
char dir_val;						// Stores direction value
char raw_message[buffer_size - 4];	// Stores message with escape sequences
char message[buffer_size - 4];		// Stores message without escape sequences


// Stores column values in hexadecimal for each letter from 'A' to 'Z'
uint8_t Alpha[27][5] =  {
		{0x3E, 0x48, 0x88, 0x48, 0x3E},	// A - 65
		{0xFE, 0x92, 0x92, 0x92, 0x6C},	// B - 66
		{0x7C, 0x82, 0x82, 0x82, 0x44},	// C - 67
		{0xFE, 0x82, 0x82, 0x82, 0x7C},	// D - 68
		{0xFE, 0x92, 0x92, 0x92, 0x82},	// E- 69
		{0xFE, 0x90, 0x90, 0x90, 0x80},	// F - 70
		{0x7C, 0x82, 0x82, 0x92, 0x5C},	// G - 71
		{0xFE, 0x10, 0x10, 0x10, 0xFE},	// H - 72
		{0x00, 0x82, 0xFE, 0x82, 0x00},	// I - 73
		{0x86, 0x82, 0xFE, 0x80, 0x80},	// J - 74
		{0xFE, 0x10, 0x28, 0x44, 0x82},	// K - 75
		{0xFE, 0x02, 0x02, 0x02, 0x02},	// L - 76
		{0xFE, 0x40, 0x20, 0x40, 0xFE},	// M - 77
		{0xFE, 0x20, 0x10, 0x08, 0xFE},	// N - 78
		{0x7C, 0x82, 0x82, 0x82, 0x7C},	// O - 79
		{0xFE, 0x90, 0x90, 0x90, 0x60},	// P - 80
		{0x7C, 0x82, 0x8A, 0x84, 0x7A},	// Q - 81
		{0xFE, 0x90, 0x98, 0x94, 0x62},	// R - 82
		{0x64, 0x92, 0x92, 0x92, 0x4C},	// S - 83
		{0xC0, 0x80, 0xFE, 0x80, 0xC0},	// T - 84
		{0xFC, 0x02, 0x02, 0x02, 0xFC},	// U - 85
		{0xF8, 0x04, 0x02, 0x04, 0xF8},	// V - 86
		{0xFE, 0x04, 0x08, 0x04, 0xFE},	// W - 87
		{0xC6, 0x28, 0x10, 0x28, 0xC6},	// X - 88
		{0xC0, 0x20, 0x1E, 0x20, 0xC0},	// Y - 89
		{0x86, 0x8A, 0x92, 0xA2, 0xC2},	// Z - 90
		{0x00, 0x00, 0x00, 0x00, 0x00}  // Blank space - 91
};


/*------------------------- END OF GLOBAL VARIABLES --------------------------*/






/**************************** FUNCTION PROTOTYPES *****************************/


/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Delay_ms
	Description:  Creates a time gap in the program flow

	Parameters (1) :
  		time = Time in milliseconds

	Return value:  None
	Function call example:  Delay_ms(200); - creates a 200 millisecond time delay

*``````````````````````````````````````````````````````````````````````````````*/
void Delay_ms (uint16_t time)
{
	HAL_Delay(time);
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Send_byte
	Description:  Sends a byte to the MAX7219 display driver.
				  Used by Send_data() & Set_LED() functions.

	Parameters (1) :
  		byte = Byte to be sent

	Return value:  None
	Function call example:  Send_byte(0xF6);

*``````````````````````````````````````````````````````````````````````````````*/
void Send_byte (uint8_t byte)
{
	for (int i=0; i<8; i++)
	{
		HAL_GPIO_WritePin (display_Port, clock_Pin, 0);  		// pull the clock pin LOW
		HAL_GPIO_WritePin (display_Port, data_Pin, byte&0x80);  // write the MS0b bit to the data pin
		byte = byte << 1;										// shift left
		HAL_GPIO_WritePin (display_Port, clock_Pin, 1);  		// pull the clock pin HIGH
	}
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Send_data
	Description:  Sends data to a specific address of MAX7219 display driver.
				  Used by Display_init() & Set_brightness() functions.

	Parameters (2) :
		address = Address byte to which data needs to be sent
  		data = Data byte to be sent to the address

	Return value:  None
	Function call example:  Send_data(0xF6, 0x3A);

*``````````````````````````````````````````````````````````````````````````````*/
void Send_data (uint8_t address, uint8_t data)
{
	HAL_GPIO_WritePin (display_Port, cs_Pin, 0);  	// pull the CS pin LOW

	for (int i=0; i<num_displays; i++)
	{
		Send_byte (address);						// send address byte
		Send_byte (data);							// send data byte
	}

	HAL_GPIO_WritePin (display_Port, cs_Pin, 1);  	// pull the CS pin HIGH
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Set_LED
	Description:  Turns ON / OFF a specific LED of the display.
				  Used by Set_col() function.

	Parameters (3) :
  		row = row value of the LED ---> input values: 1 to 8
  		col = column value of the LED ---> input values: 1 to 32
		value = 1 for ON or 0 for OFF

	Return value:  None
	Function call example:  Set_LED(8, 23, 1); - will make LED at row 8, column 23 glow

*``````````````````````````````````````````````````````````````````````````````*/
void Set_LED (uint8_t row, uint8_t col, uint8_t value)
{
  	int c = (col % 8);
  	int n = (col/8) + 1;
  	if (c==0)
  	{
  		c=8;
  		n--;
  	}

	if(value == 1) data_buffer[n-1][row-1] |= (1 << (8-c));
	else data_buffer[n-1][row-1] &= ~(1 << (8-c));

	HAL_GPIO_WritePin (display_Port, cs_Pin, 0);  // pull the CS pin LOW

	for (int i=1; i<=num_displays; i++)
	{
		if (i == n)
		{
			Send_byte (row);
			Send_byte (data_buffer[n-1][row-1]);
		}

		else
		{
			Send_byte (0);
			Send_byte (0);
		}
	}
	HAL_GPIO_WritePin (display_Port, cs_Pin, 1);  // pull the CS pin HIGH
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Set_col
	Description:  Sets an entire column of LEDs of the display
				  Used by several subsequent functions.

	Parameters (2) :
  		col = column of the display selected ---> input values: 1 to 32
		pattern = 8-bit value where 1's represent ON LEDs and 0's represent OFF LEDs

	Return value:  None
	Function call example:  Set_col(31, 0b10101010); - will make alternate LEDs of column 31 glow

*``````````````````````````````````````````````````````````````````````````````*/
void Set_col(uint8_t col, uint8_t pattern)
{
	display_col_map[col-1] = pattern;
	bool bit;

	for (int row=1; row <= 8; row++)
	{
		bit = pattern&0x80;
		Set_LED (row, col, bit);
		pattern<<=1;
	}
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Clear_display
	Description:  Clears the display
				  Used by Display_init() function

	Parameters : None

	Return value:  None
	Function call example:  Clear_display();

*``````````````````````````````````````````````````````````````````````````````*/
void Clear_display(void)
{
	for(int i=1; i<=num_displays*8; i++)
	{
		Set_col (i, 0);
	}

	for (int i=0; i<num_displays*8; i++)
	{
		display_col_map[i] = 0;
	}
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Display_init
	Description:  Initializes the display

	Parameters (1) :
		brightness = brightness of display ---> input values: 0-15 (15 is max brightness)

	Return value:  None
	Function call example:  Display_init();

*``````````````````````````````````````````````````````````````````````````````*/
void Display_init (uint8_t brightness)
{
	Send_data(0x09, 0x00);       //  No decoding mode
	Send_data(0x0b, 0x07);       //  Scan limit = 8 LEDs
	Send_data(0x0c, 0x01);       //  Power down = 0, normal mode = 1
	Send_data(0x0f, 0x00);       //  No test display
	Clear_display();
	Send_data(0x0a, brightness); // Sets display brightness intensity
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Set_brightness
	Description:  Sets the brightness of the display.
				  Used by the Scroll_message() function

	Parameters (1) :
		brightness = brightness of display ---> input values: 0-15 (15 is max brightness)

	Return value:  None
	Function call example:  Set_brightness(5);

*``````````````````````````````````````````````````````````````````````````````*/
void Set_brightness(uint8_t brightness)
{
	Send_data(0x0A, brightness);       // Sets display brightness intensity
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Write_letter
	Description:  Write a character to a specific 8x8 LED matrix of the display

	Parameters (2) :
		letter = Letter to be written ---> input values: 'A' to 'Z' & blank space (' ')
		disp_num = 8x8 display number to be written to ----> input values: 1-4 (if you have 4 8x8 displays)

	Return value:  None
	Function call example:  Write_letter('S', 4); - writes 'S' to 4th display

*``````````````````````````````````````````````````````````````````````````````*/
void Write_letter (char letter, uint8_t disp_num)
{
	int index;
	if(letter==32) index=26;
	else index = letter-65;
	int col = (disp_num-1)*8 + 2;

	for (int i=0; i<5; i++)
	{
		Set_col (col+i, Alpha[index][i]);
	}
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Shift_left
	Description:  Shifts the display LEFT
				  Used by Shift_letter() function

	Parameters : None

	Return value:  None
	Function call example:  Shift_left();

*``````````````````````````````````````````````````````````````````````````````*/
void Shift_left(void)
{
	for (int i=1; i<num_displays*8; i++)
	{
		Set_col (i, display_col_map[i]);
	}
	Set_col(num_displays*8, 0);
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Shift_right
	Description:  Shifts the display RIGHT
				  Used by Shift_letter() function

	Parameters : None

	Return value:  None
	Function call example:  Shift_right();

*``````````````````````````````````````````````````````````````````````````````*/
void Shift_right(void)
{
	for (int i=num_displays*8; i>1; i--)
	{
		Set_col (i, display_col_map[i-2]);
	}
	Set_col (1, 0);
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Shift_letter
	Description:  Shifts a single character on the display in a specified speed
				  and direction.
				  Used by Scroll_message() function

	Parameters (3) :
		letter = Letter to be written ---> input values: 'A' to 'Z' & blank space (' ')
		direction = direction of scroll ---> input values: 'L' for left, 'R' for right
		speed = scrolling speed ---> input values: 0-10 (10 is max speed)

	Return value:  None
	Function call example:  Shift_letter('M', 'L', 7);

*``````````````````````````````````````````````````````````````````````````````*/
void Shift_letter (char letter, char direction, uint32_t speed)
{
	int index;
	if(letter==32) index=26;
	else index = letter-65;

	switch (direction)
	{
		case ('L') :
			for (int i=0; i<5; i++)
			{
				Set_col (num_displays*8, Alpha[index][i]);
				Shift_left();
				HAL_Delay (200-20*speed);
			}

			Shift_left();
			break;

		case ('R') :
			for (int i=4; i>=0; i--)
			{
				Set_col (1, Alpha[index][i]);
				Shift_right();
				HAL_Delay (200-20*speed);
			}

			Shift_right();
			break;

		default :
			break;
	}
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Scroll_message
	Description:  Scrolls a message on the display with a specified speed,
				  direction and brightness

	Parameters (4) :
		*string = pointer to the message string ---> input values - either string
				  literal or pointer to string variable
		direction = scroll direction ---> input values: 'L' for left, 'R' for right
		speed = scrolling speed ---> input values: 'L','M','H' for Low, Medium & High
		brightness = display brightness ---> input values: 'L','M','H' for Low, Medium & High

	Return value:  None
	Function call example:  Scroll_message('WELCOME', 'L', 'H', 'M');
							- Scrolls "WELCOME" in LEFT direction at HIGH speed and MEDIUM brightness

*``````````````````````````````````````````````````````````````````````````````*/
void Scroll_message (char *string, char direction, char speed, char brightness)
{
	uint8_t b;
	uint8_t s;

	switch(brightness)
	{
		case 'L':
			b=0;
			break;
		case 'M':
			b=5;
			break;
		case 'H':
			b=15;
			break;
	}

	switch(speed)
	{
		case 'L':
			s=0;
			break;
		case 'M':
			s=5;
			break;
		case 'H':
			s=10;
			break;
	}

	Set_brightness(b);

	int length = strlen(string);

	if(direction =='L')
	{
		for(int i=0;i<length;i++)
		{
			Shift_letter (string[i], 'L', s);
		}
		Shift_letter (' ', 'L', s);
		Shift_letter (' ', 'L', s);
	}

	else if(direction =='R')
	{
		for(int i=length-1;i>=0;i--)
		{
			Shift_letter (string[i], 'R', s);
		}
		Shift_letter (' ', 'R', s);
		Shift_letter (' ', 'R', s);
	}
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Extract_values
	Description:  Extracts the message, direction, speed and brightness values
				  from the data buffer received from Bluetooth.
				  Used by Read_bluetooth() function

	Parameters : None

	Return value:  None
	Function call example:  Extract_values();

*``````````````````````````````````````````````````````````````````````````````*/
void Extract_values (void)
{
	dir_val = bluetooth_data[0];
	speed_val = bluetooth_data[1];
	brightness_val = bluetooth_data[2];

	for(int i=4; i<buffer_size; i++)
	{
		raw_message[i-4]=bluetooth_data[i];
	}
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Remove_escape_sequence
	Description:  Removes escape sequences at the end of the Bluetooth data buffer
				  Used by Read_bluetooth() function

	Parameters (3) :
		*pSource = Pointer to data buffer from which escape sequences are to be removed
		*pDestination = Pointer to data buffer in which cleaned buffer data is to be stored
		size = Size of source data buffer (pSource)

	Return value:  None
	Function call example:  Remove_escape_sequence(raw_message, clean_message, 20);

*``````````````````````````````````````````````````````````````````````````````*/
void Remove_escape_sequence(char *pSource ,char *pDestination, uint8_t size)
{
	while(--size)
	{
		if(*pSource != '\r' && *pSource != '\n')
		{
			*pDestination = *pSource;
			pDestination++;
		}

		pSource++;
	}
}





/*`````````````````````````````````````````````````````````````````````````````*
	Function name:  Read_bluetooth
	Description:  Receives data from Bluetooth and extracts the message, direction,
				  speed and brightness values.

	Parameters : None

	Return value:  None
	Function call example:  Read_bluetooth();

*``````````````````````````````````````````````````````````````````````````````*/
void Read_bluetooth(void)
{
	UART_Start_Receive_IT(&huart3, (uint8_t*)bluetooth_data, sizeof(bluetooth_data));
	__HAL_UART_SEND_REQ(&huart3, UART_RXDATA_FLUSH_REQUEST);

	if(uart_rx_flag)
	{
		Extract_values();

		uart_rx_flag = 0;
		Clear_display();
	}

	Remove_escape_sequence(raw_message, message, buffer_size);
}




/*------------------------ END OF FUNCTION PROTOTYPES -------------------------*/



